@@ -1,7 +1,3 @@
-#ifndef SQUID_CONFIG_H
-#include "config.h"
-#endif
-
 #ifndef _SQUID_COMPAT_H
 #define _SQUID_COMPAT_H
 
@@ -26,6 +22,20 @@
 /******************************************************/
 #include "compat/osdetect.h"
 
+/* ugly hack. But we need to set this REALLY soon in the header */
+#if _SQUID_SOLARIS_ && !defined(__GNUC__) && !defined(__GNUG__)
+#ifndef __EXTENSIONS__
+#define __EXTENSIONS__ 1
+#endif
+#ifndef _XOPEN_SOURCE
+#define _XOPEN_SOURCE 1
+#endif
+#ifndef _XOPEN_SOURCE_EXTENDED
+#define _XOPEN_SOURCE_EXTENDED 1
+#endif
+#endif
+
+
 
 /*****************************************************/
 /* FDSETSIZE is messy and needs to be done before    */
@@ -42,7 +52,7 @@
 /** On linux this must be defined to get PRId64 and friends */
 #define __STDC_FORMAT_MACROS
 
-#include "squid_types.h"
+#include "compat/types.h"
 
 /*****************************************************/
 /* per-OS hacks. One file per OS.                    */
@@ -56,6 +66,7 @@
 #include "compat/os/linux.h"
 #include "compat/os/macosx.h"
 #include "compat/os/mswin.h"
+#include "compat/os/netbsd.h"
 #include "compat/os/next.h"
 #include "compat/os/openbsd.h"
 #include "compat/os/os2.h"
@@ -75,13 +86,14 @@
 #include "compat/stdvarargs.h"
 #include "compat/assert.h"
 
-/* cstdio has a bunch of problems with 64-bit definitions */
-#include "compat/stdio.h"
 
 /*****************************************************/
 /* component-specific portabilities                  */
 /*****************************************************/
 
+/* helper debugging requires some hacks to be clean */
+#include "compat/debug.h"
+
 /* Valgrind API macros changed between two versions squid supports */
 #include "compat/valgrind.h"
 
@@ -95,5 +107,7 @@
  */
 #include "compat/GnuRegex.h"
 
+/* some functions are unsafe to be used in Squid. */
+#include "compat/unsafe.h"
 
 #endif /* _SQUID_COMPAT_H */
