@@ -1,7 +1,3 @@
-#ifndef SQUID_CONFIG_H
-#include "config.h"
-#endif
-
 #ifndef SQUID_OS_FREEBSD_H
 #define SQUID_OS_FREEBSD_H
 
@@ -27,5 +23,15 @@
 
 #define _etext etext
 
+/*
+ *   This OS has at least one version that defines these as private
+ *   kernel macros commented as being 'non-standard'.
+ *   We need to use them, much nicer than the OS-provided __u*_*[]
+ */
+//#define s6_addr8  __u6_addr.__u6_addr8
+//#define s6_addr16 __u6_addr.__u6_addr16
+#define s6_addr32 __u6_addr.__u6_addr32
+
+
 #endif /* _SQUID_FREEBSD_ */
 #endif /* SQUID_OS_FREEBSD_H */
