@@ -1,7 +1,7 @@
 #ifndef __ASYNC_IO_H__
 #define __ASYNC_IO_H__
 
-#include "config.h"
+#if USE_DISKIO_AIO
 
 #ifdef _SQUID_WIN32_
 #include "aio_win32.h"
@@ -61,4 +61,5 @@ struct _async_queue {
     int aq_numpending;		/* Num of pending ops */
 };
 
-#endif
+#endif /* USE_DISKIO_AIO */
+#endif /* __ASYNC_IO_H_ */
