@@ -1,4 +1,3 @@
-
 /*
  * $Id$
  *
@@ -34,16 +33,21 @@
 #ifndef SQUID_HTTPREQUEST_H
 #define SQUID_HTTPREQUEST_H
 
-#include "HttpMsg.h"
-#include "client_side.h"
-#include "HierarchyLogEntry.h"
-#include "HttpRequestMethod.h"
 #if USE_ADAPTATION
 #include "adaptation/History.h"
 #endif
 #if ICAP_CLIENT
 #include "adaptation/icap/History.h"
 #endif
+#include "base/CbcPointer.h"
+#include "client_side.h"
+#if USE_SQUID_EUI
+#include "eui/Eui48.h"
+#include "eui/Eui64.h"
+#endif
+#include "HierarchyLogEntry.h"
+#include "HttpMsg.h"
+#include "HttpRequestMethod.h"
 
 /*  Http Request */
 //DEAD?: extern int httpRequestHdrAllowedByName(http_hdr_type id);
@@ -87,6 +91,9 @@ public:
 
     bool conditional() const; ///< has at least one recognized If-* header
 
+    /// whether the client is likely to be able to handle a 1xx reply
+    bool canHandle1xx() const;
+
     /* Now that we care what host contains it is better off being protected. */
     /* HACK: These two methods are only inline to get around Makefile dependancies */
     /*      caused by HttpRequest being used in places it really shouldn't.        */
@@ -119,6 +126,9 @@ public:
 
     void recordLookup(const DnsLookupDetails &detail);
 
+    /// sets error detail if no earlier detail was available
+    void detailError(err_type aType, int aDetail);
+
 protected:
     void clean();
 
@@ -147,9 +157,9 @@ private:
 #endif
 
 public:
-    IpAddress host_addr;
+    Ip::Address host_addr;
 
-    AuthUserRequest *auth_user_request;
+    AuthUserRequest::Pointer auth_user_request;
 
     u_short port;
 
@@ -165,22 +175,32 @@ public:
 
     int imslen;
 
-    IpAddress client_addr;
+    Ip::Address client_addr;
 
 #if FOLLOW_X_FORWARDED_FOR
-    IpAddress indirect_client_addr;
+    Ip::Address indirect_client_addr;
 #endif /* FOLLOW_X_FORWARDED_FOR */
 
-    IpAddress my_addr;
+#if USE_SQUID_EUI
+    /* TODO these might be merged into one field if we can reliably map the EUI-48 into EUI-64
+       there are some OS differences in the upper bytes. */
+    Eui::Eui48 client_eui48;
+    Eui::Eui64 client_eui64;
+#endif
+
+    Ip::Address my_addr;
 
     HierarchyLogEntry hier;
 
     int dnsWait; ///< sum of DNS lookup delays in milliseconds, for %dt
 
     err_type errType;
+    int errDetail; ///< errType-specific detail about the transaction error
 
     char *peer_login;		/* Configured peer login:password */
 
+    char *peer_host;           /* Selected peer host*/
+
     time_t lastmod;		/* Used on refreshes */
 
     const char *vary_headers;	/* Used when varying entities are detected. Changes how the store key is calculated */
@@ -238,9 +258,16 @@ public:
         cbdataReferenceDone(pinned_connection);
     }
 
+    /// client-side conn manager, if known; used for 1xx response forwarding
+    CbcPointer<ConnStateData> clientConnection;
+
+    int64_t getRangeOffsetLimit(); /* the result of this function gets cached in rangeOffsetLimit */
+
 private:
     const char *packableURI(bool full_uri) const;
 
+    mutable int64_t rangeOffsetLimit;  /* caches the result of getRangeOffsetLimit */
+
 protected:
     virtual void packFirstLineInto(Packer * p, bool full_uri) const;
 
