@@ -5,11 +5,11 @@
 #include <libecap/common/area.h>
 #include <libecap/common/delay.h>
 #include <libecap/adapter/xaction.h>
-#include "TextException.h"
 #include "HttpRequest.h"
 #include "HttpReply.h"
 #include "SquidTime.h"
 #include "adaptation/ecap/XactionRep.h"
+#include "base/TextException.h"
 
 CBDATA_NAMESPACED_CLASS_INIT(Adaptation::Ecap::XactionRep, XactionRep);
 
@@ -446,7 +446,7 @@ Adaptation::Ecap::XactionRep::status() const
             buf.append(" A.", 3);
     }
 
-    buf.Printf(" ecapx%d]", id);
+    buf.Printf(" %s%u]", id.Prefix, id.value);
 
     buf.terminate();
 
