@@ -43,24 +43,8 @@
  */
 
 #include "squid.h"
-#include "clientStream.h"
-#include "client_side_request.h"
-#include "auth/UserRequest.h"
-#include "HttpRequest.h"
-#include "ProtoPort.h"
 #include "acl/FilledChecklist.h"
 #include "acl/Gadgets.h"
-#include "client_side.h"
-#include "client_side_reply.h"
-#include "Store.h"
-#include "HttpReply.h"
-#include "MemObject.h"
-#include "ClientRequestContext.h"
-#include "SquidTime.h"
-#include "wordlist.h"
-#include "inet_pton.h"
-#include "fde.h"
-
 #if USE_ADAPTATION
 #include "adaptation/AccessCheck.h"
 #include "adaptation/Iterator.h"
@@ -68,9 +52,29 @@
 #if ICAP_CLIENT
 #include "adaptation/icap/History.h"
 #endif
-//static void adaptationAclCheckDoneWrapper(Adaptation::ServicePointer service, void *data);
 #endif
-
+#include "auth/UserRequest.h"
+#include "clientStream.h"
+#include "client_side.h"
+#include "client_side_reply.h"
+#include "client_side_request.h"
+#include "ClientRequestContext.h"
+#include "comm/Write.h"
+#include "compat/inet_pton.h"
+#include "fde.h"
+#include "HttpReply.h"
+#include "HttpRequest.h"
+#include "ip/QosConfig.h"
+#include "log/Tokens.h"
+#include "MemObject.h"
+#include "ProtoPort.h"
+#include "Store.h"
+#include "SquidTime.h"
+#include "wordlist.h"
+#include "err_detail_type.h"
+#if USE_SSL
+#include "ssl/support.h"
+#endif
 
 
 #if LINGERING_CLOSE
@@ -105,9 +109,6 @@ ClientRequestContext::operator delete (void *address)
 /* Local functions */
 /* other */
 static void clientAccessCheckDoneWrapper(int, void *);
-#if USE_SSL
-static void sslBumpAccessCheckDoneWrapper(int, void *);
-#endif
 static int clientHierarchical(ClientHttpRequest * http);
 static void clientInterpretRequestHeaders(ClientHttpRequest * http);
 static RH clientRedirectDoneWrapper;
@@ -136,9 +137,6 @@ ClientRequestContext::ClientRequestContext(ClientHttpRequest *anHttp) : http(cbd
     redirect_done = false;
     no_cache_done = false;
     interpreted_req_hdrs = false;
-#if USE_SSL
-    sslBumpCheckDone = false;
-#endif
     debugs(85,3, HERE << this << " ClientRequestContext constructed");
 }
 
@@ -172,9 +170,6 @@ ClientHttpRequest::ClientHttpRequest(ConnStateData * aConn) :
 #if USE_ADAPTATION
     request_satisfaction_mode = false;
 #endif
-#if USE_SSL
-    sslBumpNeed = needUnknown;
-#endif
 }
 
 /*
@@ -189,7 +184,7 @@ ClientHttpRequest::onlyIfCached()const
            EBIT_TEST(request->cache_control->mask, CC_ONLY_IF_CACHED);
 }
 
-/**
+/*
  * This function is designed to serve a fairly specific purpose.
  * Occasionally our vBNS-connected caches can talk to each other, but not
  * the rest of the world.  Here we try to detect frequent failures which
@@ -201,25 +196,19 @@ ClientHttpRequest::onlyIfCached()const
  *
  * Duane W., Sept 16, 1996
  */
+
+#define FAILURE_MODE_TIME 300
+
 static void
 checkFailureRatio(err_type etype, hier_code hcode)
 {
-    // Can be set at compile time with -D compiler flag
-#ifndef FAILURE_MODE_TIME
-#define FAILURE_MODE_TIME 300
-#endif
-
-    if (hcode == HIER_NONE)
-        return;
-
-    // don't bother when ICP is disabled.
-    if (Config.Port.icp <= 0)
-        return;
-
     static double magic_factor = 100.0;
     double n_good;
     double n_bad;
 
+    if (hcode == HIER_NONE)
+        return;
+
     n_good = magic_factor / (1.0 + request_failure_ratio);
 
     n_bad = magic_factor - n_good;
@@ -247,10 +236,10 @@ checkFailureRatio(err_type etype, hier_code hcode)
     if (request_failure_ratio < 1.0)
         return;
 
-    debugs(33, DBG_CRITICAL, "WARNING: Failure Ratio at "<< std::setw(4)<<
+    debugs(33, 0, "Failure Ratio at "<< std::setw(4)<<
            std::setprecision(3) << request_failure_ratio);
 
-    debugs(33, DBG_CRITICAL, "WARNING: ICP going into HIT-only mode for " <<
+    debugs(33, 0, "Going into hit-only-mode for " <<
            FAILURE_MODE_TIME / 60 << " minutes...");
 
     hit_only_mode_until = squid_curtime + FAILURE_MODE_TIME;
@@ -458,7 +447,7 @@ clientFollowXForwardedForCheck(int answer, void *data)
         const char *p;
         const char *asciiaddr;
         int l;
-        IpAddress addr;
+        Ip::Address addr;
         p = request->x_forwarded_for_iterator.termedBuf();
         l = request->x_forwarded_for_iterator.size();
 
@@ -634,16 +623,16 @@ ClientRequestContext::clientAccessCheckDone(int answer)
         clientStreamNode *node = (clientStreamNode *)http->client_stream.tail->prev->data;
         clientReplyContext *repContext = dynamic_cast<clientReplyContext *>(node->data.getRaw());
         assert (repContext);
-        IpAddress tmpnoaddr;
+        Ip::Address tmpnoaddr;
         tmpnoaddr.SetNoAddr();
         repContext->setReplyToError(page_id, status,
                                     http->request->method, NULL,
                                     http->getConn() != NULL ? http->getConn()->peer : tmpnoaddr,
                                     http->request,
                                     NULL,
-                                    http->getConn() != NULL && http->getConn()->auth_user_request ?
+                                    http->getConn() != NULL && http->getConn()->auth_user_request != NULL ?
                                     http->getConn()->auth_user_request : http->request->auth_user_request);
-        http->getConn()->flags.readMoreRequests = true; // resume any pipeline reads.
+
         node = (clientStreamNode *)http->client_stream.tail->data;
         clientStreamRead(node, http, node->readBuffer);
         return;
@@ -775,6 +764,67 @@ clientHierarchical(ClientHttpRequest * http)
 }
 
 
+static void
+clientCheckPinning(ClientHttpRequest * http)
+{
+    HttpRequest *request = http->request;
+    HttpHeader *req_hdr = &request->header;
+    ConnStateData *http_conn = http->getConn();
+
+    /* Internal requests such as those from ESI includes may be without
+     * a client connection
+     */
+    if (!http_conn)
+        return;
+
+    request->flags.connection_auth_disabled = http_conn->port->connection_auth_disabled;
+    if (!request->flags.connection_auth_disabled) {
+        if (http_conn->pinning.fd != -1) {
+            if (http_conn->pinning.auth) {
+                request->flags.connection_auth = 1;
+                request->flags.auth = 1;
+            } else {
+                request->flags.connection_proxy_auth = 1;
+            }
+            request->setPinnedConnection(http_conn);
+        }
+    }
+
+    /* check if connection auth is used, and flag as candidate for pinning
+     * in such case.
+     * Note: we may need to set flags.connection_auth even if the connection
+     * is already pinned if it was pinned earlier due to proxy auth
+     */
+    if (!request->flags.connection_auth) {
+        if (req_hdr->has(HDR_AUTHORIZATION) || req_hdr->has(HDR_PROXY_AUTHORIZATION)) {
+            HttpHeaderPos pos = HttpHeaderInitPos;
+            HttpHeaderEntry *e;
+            int may_pin = 0;
+            while ((e = req_hdr->getEntry(&pos))) {
+                if (e->id == HDR_AUTHORIZATION || e->id == HDR_PROXY_AUTHORIZATION) {
+                    const char *value = e->value.rawBuf();
+                    if (strncasecmp(value, "NTLM ", 5) == 0
+                            ||
+                            strncasecmp(value, "Negotiate ", 10) == 0
+                            ||
+                            strncasecmp(value, "Kerberos ", 9) == 0) {
+                        if (e->id == HDR_AUTHORIZATION) {
+                            request->flags.connection_auth = 1;
+                            may_pin = 1;
+                        } else {
+                            request->flags.connection_proxy_auth = 1;
+                            may_pin = 1;
+                        }
+                    }
+                }
+            }
+            if (may_pin && !request->pinnedConnection()) {
+                request->setPinnedConnection(http->getConn());
+            }
+        }
+    }
+}
+
 static void
 clientInterpretRequestHeaders(ClientHttpRequest * http)
 {
@@ -831,7 +881,7 @@ clientInterpretRequestHeaders(ClientHttpRequest * http)
     }
 
     if (no_cache) {
-#if HTTP_VIOLATIONS
+#if USE_HTTP_VIOLATIONS
 
         if (Config.onoff.reload_into_ims)
             request->flags.nocache_hack = 1;
@@ -878,59 +928,7 @@ clientInterpretRequestHeaders(ClientHttpRequest * http)
     if (req_hdr->has(HDR_AUTHORIZATION))
         request->flags.auth = 1;
 
-    ConnStateData *http_conn = http->getConn();
-    if (http_conn) {
-        request->flags.connection_auth_disabled = http_conn->port->connection_auth_disabled;
-        if (!request->flags.connection_auth_disabled) {
-            if (http_conn->pinning.fd != -1) {
-                if (http_conn->pinning.auth) {
-                    request->flags.connection_auth = 1;
-                    request->flags.auth = 1;
-                } else {
-                    request->flags.connection_proxy_auth = 1;
-                }
-                request->setPinnedConnection(http_conn);
-            }
-        }
-    } else {
-        // internal requests and ESI don't have client conn.
-        request->flags.connection_auth_disabled = 1;
-    }
-
-    /* check if connection auth is used, and flag as candidate for pinning
-     * in such case.
-     * Note: we may need to set flags.connection_auth even if the connection
-     * is already pinned if it was pinned earlier due to proxy auth
-     */
-    if (!request->flags.connection_auth) {
-        if (req_hdr->has(HDR_AUTHORIZATION) || req_hdr->has(HDR_PROXY_AUTHORIZATION)) {
-            HttpHeaderPos pos = HttpHeaderInitPos;
-            HttpHeaderEntry *e;
-            int may_pin = 0;
-            while ((e = req_hdr->getEntry(&pos))) {
-                if (e->id == HDR_AUTHORIZATION || e->id == HDR_PROXY_AUTHORIZATION) {
-                    const char *value = e->value.rawBuf();
-                    if (strncasecmp(value, "NTLM ", 5) == 0
-                            ||
-                            strncasecmp(value, "Negotiate ", 10) == 0
-                            ||
-                            strncasecmp(value, "Kerberos ", 9) == 0) {
-                        if (e->id == HDR_AUTHORIZATION) {
-                            request->flags.connection_auth = 1;
-                            may_pin = 1;
-                        } else {
-                            request->flags.connection_proxy_auth = 1;
-                            may_pin = 1;
-                        }
-                    }
-                }
-            }
-            if (may_pin && !request->pinnedConnection()) {
-                request->setPinnedConnection(http->getConn());
-            }
-        }
-    }
-
+    clientCheckPinning(http);
 
     if (request->login[0] != '\0')
         request->flags.auth = 1;
@@ -938,7 +936,7 @@ clientInterpretRequestHeaders(ClientHttpRequest * http)
     if (req_hdr->has(HDR_VIA)) {
         String s = req_hdr->getList(HDR_VIA);
         /*
-         * ThisCache cannot be a member of Via header, "1.0 ThisCache" can.
+         * ThisCache cannot be a member of Via header, "1.1 ThisCache" can.
          * Note ThisCache2 has a space prepended to the hostname so we don't
          * accidentally match super-domains.
          */
@@ -949,7 +947,7 @@ clientInterpretRequestHeaders(ClientHttpRequest * http)
             request->flags.loopdetect = 1;
         }
 
-#if FORW_VIA_DB
+#if USE_FORW_VIA_DB
         fvdbCountVia(s.termedBuf());
 
 #endif
@@ -957,22 +955,7 @@ clientInterpretRequestHeaders(ClientHttpRequest * http)
         s.clean();
     }
 
-    /**
-     \todo  --enable-useragent-log and --enable-referer-log. We should
-            probably drop those two as the custom log formats accomplish pretty much the same thing..
-    */
-#if USE_USERAGENT_LOG
-    if ((str = req_hdr->getStr(HDR_USER_AGENT)))
-        logUserAgent(fqdnFromAddr(http->getConn()->log_addr), str);
-
-#endif
-#if USE_REFERER_LOG
-
-    if ((str = req_hdr->getStr(HDR_REFERER)))
-        logReferer(fqdnFromAddr(http->getConn()->log_addr), str, http->log_uri);
-
-#endif
-#if FORW_VIA_DB
+#if USE_FORW_VIA_DB
 
     if (req_hdr->has(HDR_X_FORWARDED_FOR)) {
         String s = req_hdr->getList(HDR_X_FORWARDED_FOR);
@@ -1010,6 +993,7 @@ clientRedirectDoneWrapper(void *data, char *result)
 void
 ClientRequestContext::clientRedirectDone(char *result)
 {
+    HttpRequest *new_request = NULL;
     HttpRequest *old_request = http->request;
     debugs(85, 5, "clientRedirectDone: '" << http->uri << "' result=" << (result ? result : "NULL"));
     assert(redirect_state == REDIRECT_PENDING);
@@ -1035,35 +1019,41 @@ ClientRequestContext::clientRedirectDone(char *result)
                     debugs(85, DBG_CRITICAL, "ERROR: URL-rewrite produces invalid 303 redirect Location: " << result);
             }
         } else if (strcmp(result, http->uri)) {
-            // XXX: validate the URL properly *without* generating a whole new request object right here.
-            // XXX: the clone() should be done only AFTER we know the new URL is valid.
-            HttpRequest *new_request = old_request->clone();
-            if (urlParse(old_request->method, result, new_request)) {
-                debugs(61,2, HERE << "URL-rewriter diverts URL from " << urlCanonical(old_request) << " to " << urlCanonical(new_request));
-
-                // update the new request to flag the re-writing was done on it
-                new_request->flags.redirected = 1;
-
-                // unlink bodypipe from the old request. Not needed there any longer.
-                if (old_request->body_pipe != NULL) {
-                    old_request->body_pipe = NULL;
-                    debugs(61,2, HERE << "URL-rewriter diverts body_pipe " << new_request->body_pipe <<
-                           " from request " << old_request << " to " << new_request);
-                }
-
-                // update the current working ClientHttpRequest fields
-                safe_free(http->uri);
-                http->uri = xstrdup(urlCanonical(new_request));
-                HTTPMSGUNLOCK(old_request);
-                http->request = HTTPMSGLOCK(new_request);
-            } else {
+            if (!(new_request = HttpRequest::CreateFromUrlAndMethod(result, old_request->method)))
                 debugs(85, DBG_CRITICAL, "ERROR: URL-rewrite produces invalid request: " <<
                        old_request->method << " " << result << " HTTP/1.1");
-                delete new_request;
-            }
         }
     }
 
+    if (new_request) {
+        safe_free(http->uri);
+        http->uri = xstrdup(urlCanonical(new_request));
+        new_request->http_ver = old_request->http_ver;
+        new_request->header.append(&old_request->header);
+        new_request->client_addr = old_request->client_addr;
+#if FOLLOW_X_FORWARDED_FOR
+        new_request->indirect_client_addr = old_request->indirect_client_addr;
+#endif /* FOLLOW_X_FORWARDED_FOR */
+        new_request->my_addr = old_request->my_addr;
+        new_request->flags = old_request->flags;
+        new_request->flags.redirected = 1;
+        new_request->auth_user_request = old_request->auth_user_request;
+
+        if (old_request->body_pipe != NULL) {
+            new_request->body_pipe = old_request->body_pipe;
+            old_request->body_pipe = NULL;
+            debugs(61,2, HERE << "URL-rewriter diverts body_pipe " << new_request->body_pipe <<
+                   " from request " << old_request << " to " << new_request);
+        }
+
+        new_request->content_length = old_request->content_length;
+        new_request->extacl_user = old_request->extacl_user;
+        new_request->extacl_passwd = old_request->extacl_passwd;
+        new_request->flags.proxy_keepalive = old_request->flags.proxy_keepalive;
+        HTTPMSGUNLOCK(old_request);
+        http->request = HTTPMSGLOCK(new_request);
+    }
+
     /* FIXME PIPELINE: This is innacurate during pipelining */
 
     if (http->getConn() != NULL)
@@ -1108,45 +1098,6 @@ ClientRequestContext::checkNoCacheDone(int answer)
     http->doCallouts();
 }
 
-#if USE_SSL
-bool
-ClientRequestContext::sslBumpAccessCheck()
-{
-    if (http->request->method == METHOD_CONNECT &&
-            Config.accessList.ssl_bump && http->getConn()->port->sslBump) {
-        debugs(85, 5, HERE << "SslBump possible, checking ACL");
-
-        ACLFilledChecklist *acl_checklist = clientAclChecklistCreate(Config.accessList.ssl_bump, http);
-        acl_checklist->nonBlockingCheck(sslBumpAccessCheckDoneWrapper, this);
-        return true;
-    } else {
-        http->sslBumpNeeded(false);
-        return false;
-    }
-}
-
-/**
- * A wrapper function to use the ClientRequestContext::sslBumpAccessCheckDone method
- * as ACLFilledChecklist callback
- */
-static void
-sslBumpAccessCheckDoneWrapper(int answer, void *data)
-{
-    ClientRequestContext *calloutContext = static_cast<ClientRequestContext *>(data);
-
-    if (!calloutContext->httpStateIsValid())
-        return;
-    calloutContext->sslBumpAccessCheckDone(answer == ACCESS_ALLOWED);
-}
-
-void
-ClientRequestContext::sslBumpAccessCheckDone(bool doSslBump)
-{
-    http->sslBumpNeeded(doSslBump);
-    http->doCallouts();
-}
-#endif
-
 /*
  * Identify requests that do not go through the store and client side stream
  * and forward them to the appropriate location. All other requests, request
@@ -1157,13 +1108,14 @@ ClientHttpRequest::processRequest()
 {
     debugs(85, 4, "clientProcessRequest: " << RequestMethodStr(request->method) << " '" << uri << "'");
 
-    if (request->method == METHOD_CONNECT && !redirect.status) {
 #if USE_SSL
-        if (sslBumpNeeded()) {
-            sslBumpStart();
-            return;
-        }
+    if (request->method == METHOD_CONNECT && sslBumpNeeded()) {
+        sslBumpStart();
+        return;
+    }
 #endif
+
+    if (request->method == METHOD_CONNECT && !redirect.status) {
         logType = LOG_TCP_MISS;
         getConn()->stopReading(); // tunnels read for themselves
         tunnelStart(this, &out.size, &al.http.code);
@@ -1190,18 +1142,19 @@ ClientHttpRequest::httpStart()
 
 #if USE_SSL
 
+// determines whether we should bump the CONNECT request
 bool
 ClientHttpRequest::sslBumpNeeded() const
 {
-    assert(sslBumpNeed != needUnknown);
-    return (sslBumpNeed == needConfirmed);
-}
+    if (!getConn()->port->sslBump || !Config.accessList.ssl_bump)
+        return false;
 
-void
-ClientHttpRequest::sslBumpNeeded(bool isNeeded)
-{
-    debugs(83, 3, HERE << "sslBump required: "<< (isNeeded ? "Yes" : "No"));
-    sslBumpNeed = (isNeeded ? needConfirmed : needNot);
+    debugs(85, 5, HERE << "SslBump possible, checking ACL");
+
+    ACLFilledChecklist check(Config.accessList.ssl_bump, request, NULL);
+    check.src_addr = request->client_addr;
+    check.my_addr = request->my_addr;
+    return check.fastCheck() == 1;
 }
 
 // called when comm_write has completed
@@ -1223,8 +1176,7 @@ ClientHttpRequest::sslBumpEstablish(comm_err_t errflag)
         return;
 
     if (errflag) {
-        debugs(85, 3, HERE << "CONNECT response failure in SslBump: " << errflag);
-        comm_close(getConn()->fd);
+        getConn()->startClosing("CONNECT response failure in SslBump");
         return;
     }
 
@@ -1242,9 +1194,10 @@ ClientHttpRequest::sslBumpStart()
 
     // TODO: Unify with tunnel.cc and add a Server(?) header
     static const char *const conn_established =
-        "HTTP/1.0 200 Connection established\r\n\r\n";
-    comm_write(fd, conn_established, strlen(conn_established),
-               &SslBumpEstablish, this, NULL);
+        "HTTP/1.1 200 Connection established\r\n\r\n";
+    AsyncCall::Pointer call = commCbCall(85, 5, "ClientSocketContext::sslBumpEstablish",
+                                         CommIoCbPtrFun(&SslBumpEstablish, this));
+    Comm::Write(fd, conn_established, strlen(conn_established), call, NULL);
 }
 
 #endif
@@ -1313,7 +1266,8 @@ ClientHttpRequest::loggingEntry(StoreEntry *newEntry)
  * the callout.  This is strictly for convenience.
  */
 
-extern int aclMapTOS (acl_tos * head, ACLChecklist * ch);
+extern tos_t aclMapTOS (acl_tos * head, ACLChecklist * ch);
+extern nfmark_t aclMapNfmark (acl_nfmark * head, ACLChecklist * ch);
 
 void
 ClientHttpRequest::doCallouts()
@@ -1376,26 +1330,29 @@ ClientHttpRequest::doCallouts()
         }
     }
 
-    if (!calloutContext->clientside_tos_done) {
-        calloutContext->clientside_tos_done = true;
+    if (!calloutContext->tosToClientDone) {
+        calloutContext->tosToClientDone = true;
         if (getConn() != NULL) {
             ACLFilledChecklist ch(NULL, request, NULL);
             ch.src_addr = request->client_addr;
             ch.my_addr = request->my_addr;
-            int tos = aclMapTOS(Config.accessList.clientside_tos, &ch);
+            tos_t tos = aclMapTOS(Ip::Qos::TheConfig.tosToClient, &ch);
             if (tos)
-                comm_set_tos(getConn()->fd, tos);
+                Ip::Qos::setSockTos(getConn()->fd, tos);
         }
     }
 
-#if USE_SSL
-    if (!calloutContext->sslBumpCheckDone) {
-        calloutContext->sslBumpCheckDone = true;
-        if (calloutContext->sslBumpAccessCheck())
-            return;
-        /* else no ssl bump required*/
+    if (!calloutContext->nfmarkToClientDone) {
+        calloutContext->nfmarkToClientDone = true;
+        if (getConn() != NULL) {
+            ACLFilledChecklist ch(NULL, request, NULL);
+            ch.src_addr = request->client_addr;
+            ch.my_addr = request->my_addr;
+            nfmark_t mark = aclMapNfmark(Ip::Qos::TheConfig.nfmarkToClient, &ch);
+            if (mark)
+                Ip::Qos::setSockNfmark(getConn()->fd, mark);
+        }
     }
-#endif
 
     cbdataReferenceDone(calloutContext->http);
     delete calloutContext;
@@ -1415,7 +1372,7 @@ ClientHttpRequest::doCallouts()
 #endif
 }
 
-#ifndef _USE_INLINE_
+#if !_USE_INLINE_
 #include "client_side_request.cci"
 #endif
 
@@ -1491,16 +1448,7 @@ ClientHttpRequest::noteAdaptationQueryAbort(bool final)
 {
     clearAdaptation(virginHeadSource);
     assert(!adaptedBodySource);
-    handleAdaptationFailure(!final);
-}
-
-void
-ClientHttpRequest::resumeBodyStorage()
-{
-    if (!adaptedBodySource)
-        return;
-
-    noteMoreBodyDataAvailable(adaptedBodySource);
+    handleAdaptationFailure(ERR_DETAIL_CLT_REQMOD_ABORT, !final);
 }
 
 void
@@ -1509,28 +1457,12 @@ ClientHttpRequest::noteMoreBodyDataAvailable(BodyPipe::Pointer)
     assert(request_satisfaction_mode);
     assert(adaptedBodySource != NULL);
 
-    if (size_t contentSize = adaptedBodySource->buf().contentSize()) {
-        const size_t spaceAvailable = storeEntry()->bytesWanted(Range<size_t>(0,contentSize));
-
-        if (spaceAvailable < contentSize ) {
-            // No or partial body data consuming
-            typedef NullaryMemFunT<ClientHttpRequest> Dialer;
-            AsyncCall::Pointer call = asyncCall(93, 5, "ClientHttpRequest::resumeBodyStorage",
-                                                Dialer(this, &ClientHttpRequest::resumeBodyStorage));
-            storeEntry()->deferProducer(call);
-        }
-
-        if (!spaceAvailable)
-            return;
-
-        if (spaceAvailable < contentSize )
-            contentSize = spaceAvailable;
-
+    if (const size_t contentSize = adaptedBodySource->buf().contentSize()) {
         BodyPipeCheckout bpc(*adaptedBodySource);
-        const StoreIOBuffer ioBuf(&bpc.buf, request_satisfaction_offset, contentSize);
+        const StoreIOBuffer ioBuf(&bpc.buf, request_satisfaction_offset);
         storeEntry()->write(ioBuf);
-        // assume StoreEntry::write() writes the entire ioBuf
-        request_satisfaction_offset += ioBuf.length;
+        // assume can write everything
+        request_satisfaction_offset += contentSize;
         bpc.buf.consume(contentSize);
         bpc.checkIn();
     }
@@ -1544,9 +1476,13 @@ void
 ClientHttpRequest::noteBodyProductionEnded(BodyPipe::Pointer)
 {
     assert(!virginHeadSource);
-    // should we end request satisfaction now?
-    if (adaptedBodySource != NULL && adaptedBodySource->exhausted())
+    if (adaptedBodySource != NULL) { // did not end request satisfaction yet
+        // We do not expect more because noteMoreBodyDataAvailable always
+        // consumes everything. We do not even have a mechanism to consume
+        // leftovers after noteMoreBodyDataAvailable notifications seize.
+        assert(adaptedBodySource->exhausted());
         endRequestSatisfaction();
+    }
 }
 
 void
@@ -1565,11 +1501,20 @@ ClientHttpRequest::noteBodyProducerAborted(BodyPipe::Pointer)
 {
     assert(!virginHeadSource);
     stopConsumingFrom(adaptedBodySource);
-    handleAdaptationFailure();
+
+    debugs(85,3, HERE << "REQMOD body production failed");
+    if (request_satisfaction_mode) { // too late to recover or serve an error
+        request->detailError(ERR_ICAP_FAILURE, ERR_DETAIL_CLT_REQMOD_RESP_BODY);
+        const int fd = getConn()->fd;
+        Must(fd >= 0);
+        comm_close(fd); // drastic, but we may be writing a response already
+    } else {
+        handleAdaptationFailure(ERR_DETAIL_CLT_REQMOD_REQ_BODY);
+    }
 }
 
 void
-ClientHttpRequest::handleAdaptationFailure(bool bypassable)
+ClientHttpRequest::handleAdaptationFailure(int errDetail, bool bypassable)
 {
     debugs(85,3, HERE << "handleAdaptationFailure(" << bypassable << ")");
 
@@ -1593,16 +1538,17 @@ ClientHttpRequest::handleAdaptationFailure(bool bypassable)
     // The original author of the code also wanted to pass an errno to
     // setReplyToError, but it seems unlikely that the errno reflects the
     // true cause of the error at this point, so I did not pass it.
-    IpAddress noAddr;
+    Ip::Address noAddr;
     noAddr.SetNoAddr();
     ConnStateData * c = getConn();
     repContext->setReplyToError(ERR_ICAP_FAILURE, HTTP_INTERNAL_SERVER_ERROR,
                                 request->method, NULL,
                                 (c != NULL ? c->peer : noAddr), request, NULL,
-                                (c != NULL && c->auth_user_request ?
+                                (c != NULL && c->auth_user_request != NULL ?
                                  c->auth_user_request : request->auth_user_request));
-    c->flags.readMoreRequests = true;
-    c->expectNoForwarding();
+
+    request->detailError(ERR_ICAP_FAILURE, errDetail);
+
     node = (clientStreamNode *)client_stream.tail->data;
     clientStreamRead(node, this, node->readBuffer);
 }
