@@ -1,7 +1,6 @@
 #ifndef SQUID_LISTENERSTATEDATA_H
 #define SQUID_LISTENERSTATEDATA_H
 
-#include "config.h"
 #include "base/AsyncCall.h"
 #include "comm.h"
 #if HAVE_MAP
@@ -23,11 +22,11 @@ public:
 
     void subscribe(AsyncCall::Pointer &call);
     void acceptNext();
-    void notify(int newfd, comm_err_t, int xerrno, const ConnectionDetail &);
+    void notify(int newfd, comm_err_t flag, const ConnectionDetail &details);
 
     int fd;
 
-    /// errno code if any happened so far.
+    /// errno code of the last accept() or listen() action if one occurred.
     int errcode;
 
     /// whether this socket is delayed and on the AcceptLimiter queue.
@@ -50,6 +49,6 @@ private:
     void setListen();
 };
 
-}; // namespace Comm
+} // namespace Comm
 
 #endif /* SQUID_LISTENERSTATEDATA_H */
