@@ -33,7 +33,7 @@
 
 #include "squid.h"
 #include "comm_select.h"
-#include "CacheManager.h"
+#include "mgr/Registration.h"
 #include "SquidTime.h"
 
 #if USE_SELECT_WIN32
@@ -380,11 +380,11 @@ comm_select(int msec)
 
         maxfd = Biggest_FD + 1;
 
-        xmemcpy(&readfds, &global_readfds, sizeof(global_readfds));
+        memcpy(&readfds, &global_readfds, sizeof(global_readfds));
 
-        xmemcpy(&writefds, &global_writefds, sizeof(global_writefds));
+        memcpy(&writefds, &global_writefds, sizeof(global_writefds));
 
-        xmemcpy(&errfds, &global_writefds, sizeof(global_writefds));
+        memcpy(&errfds, &global_writefds, sizeof(global_writefds));
 
         /* remove stalled FDs, and deal with pending descriptors */
         pending = 0;
@@ -685,10 +685,9 @@ comm_select_dns_incoming(void)
 static void
 commSelectRegisterWithCacheManager(void)
 {
-    CacheManager::GetInstance()->
-    registerAction("comm_select_incoming",
-                   "comm_incoming() stats",
-                   commIncomingStats, 0, 1);
+    Mgr::RegisterAction("comm_select_incoming",
+                        "comm_incoming() stats",
+                        commIncomingStats, 0, 1);
 }
 
 void
