@@ -37,6 +37,7 @@
 #include "htcp.h"
 #include "acl/FilledChecklist.h"
 #include "acl/Acl.h"
+#include "ip/tools.h"
 #include "SquidTime.h"
 #include "Store.h"
 #include "StoreClient.h"
@@ -46,6 +47,24 @@
 #include "http.h"
 #include "icmp/net_db.h"
 #include "AccessLogEntry.h"
+#include "ipc/StartListening.h"
+
+/// dials htcpIncomingConnectionOpened call
+class HtcpListeningStartedDialer: public CallDialer,
+        public Ipc::StartListeningCb
+{
+public:
+    typedef void (*Handler)(int fd, int errNo);
+    HtcpListeningStartedDialer(Handler aHandler): handler(aHandler) {}
+
+    virtual void print(std::ostream &os) const { startPrint(os) << ')'; }
+
+    virtual bool canDial(AsyncCall &) const { return true; }
+    virtual void dial(AsyncCall &) { (handler)(fd, errNo); }
+
+public:
+    Handler handler;
+};
 
 typedef struct _Countstr Countstr;
 
@@ -62,18 +81,18 @@ typedef struct _htcpStuff htcpStuff;
 typedef struct _htcpDetail htcpDetail;
 
 struct _Countstr {
-    u_int16_t length;
+    uint16_t length;
     char *text;
 };
 
 struct _htcpHeader {
-    u_int16_t length;
+    uint16_t length;
     u_char major;
     u_char minor;
 };
 
 struct _htcpDataHeaderSquid {
-    u_int16_t length;
+    uint16_t length;
 
 #if !WORDS_BIGENDIAN
     unsigned int opcode:4;
@@ -93,41 +112,41 @@ struct _htcpDataHeaderSquid {
     unsigned int reserved:6;
 #endif
 
-    u_int32_t msg_id;
+    uint32_t msg_id;
 };
 
 struct _htcpDataHeader {
-    u_int16_t length;
+    uint16_t length;
 
 #if WORDS_BIGENDIAN
-u_int8_t opcode:
+uint8_t opcode:
     4;
-u_int8_t response:
+uint8_t response:
     4;
 #else
-u_int8_t response:
+uint8_t response:
     4;
-u_int8_t opcode:
+uint8_t opcode:
     4;
 #endif
 
 #if WORDS_BIGENDIAN
-u_int8_t reserved:
+uint8_t reserved:
     6;
-u_int8_t F1:
+uint8_t F1:
     1;
-u_int8_t RR:
+uint8_t RR:
     1;
 #else
-u_int8_t RR:
+uint8_t RR:
     1;
-u_int8_t F1:
+uint8_t F1:
     1;
-u_int8_t reserved:
+uint8_t reserved:
     6;
 #endif
 
-    u_int32_t msg_id;
+    uint32_t msg_id;
 };
 
 /* RR == 0 --> F1 = RESPONSE DESIRED FLAG */
@@ -136,7 +155,7 @@ u_int8_t reserved:
 /* RR == 1 --> RESPONSE */
 
 struct _htcpAuthHeader {
-    u_int16_t length;
+    uint16_t length;
     time_t sig_time;
     time_t sig_expire;
     Countstr key_name;
@@ -153,8 +172,8 @@ public:
     void checkHit();
     void checkedHit(StoreEntry *e);
 
-    void setFrom (IpAddress &from);
-    void setDataHeader (htcpDataHeader *);
+    void setFrom(Ip::Address &from);
+    void setDataHeader(htcpDataHeader *);
     char *method;
     char *uri;
     char *version;
@@ -164,7 +183,7 @@ public:
 private:
     HttpRequest *checkHitRequest;
 
-    IpAddress from; // was a ptr. return to such IFF needed. otherwise copy should do.
+    Ip::Address from; // was a ptr. return to such IFF needed. otherwise copy should do.
     htcpDataHeader *dhdr;
 };
 
@@ -182,7 +201,7 @@ struct _htcpStuff {
     int f1;
     int response;
     int reason;
-    u_int32_t msg_id;
+    uint32_t msg_id;
     htcpSpecifier S;
     htcpDetail D;
 };
@@ -225,14 +244,16 @@ enum {
     RR_RESPONSE
 };
 
-static u_int32_t msg_id_counter = 0;
+static void htcpIncomingConnectionOpened(int fd, int errNo);
+static uint32_t msg_id_counter = 0;
+
 static int htcpInSocket = -1;
 static int htcpOutSocket = -1;
 #define N_QUERIED_KEYS 8192
-static u_int32_t queried_id[N_QUERIED_KEYS];
+static uint32_t queried_id[N_QUERIED_KEYS];
 static cache_key queried_keys[N_QUERIED_KEYS][SQUID_MD5_DIGEST_LENGTH];
 
-static IpAddress queried_addr[N_QUERIED_KEYS];
+static Ip::Address queried_addr[N_QUERIED_KEYS];
 static MemAllocator *htcpDetailPool = NULL;
 
 static int old_squid_format = 0;
@@ -251,26 +272,26 @@ static ssize_t htcpBuildTstOpData(char *buf, size_t buflen, htcpStuff * stuff);
 static void htcpFreeSpecifier(htcpSpecifier * s);
 static void htcpFreeDetail(htcpDetail * s);
 
-static void htcpHandleMsg(char *buf, int sz, IpAddress &from);
+static void htcpHandleMsg(char *buf, int sz, Ip::Address &from);
 
-static void htcpLogHtcp(IpAddress &, int, log_type, const char *);
-static void htcpHandleMon(htcpDataHeader *, char *buf, int sz, IpAddress &from);
+static void htcpLogHtcp(Ip::Address &, int, log_type, const char *);
+static void htcpHandleMon(htcpDataHeader *, char *buf, int sz, Ip::Address &from);
 
-static void htcpHandleNop(htcpDataHeader *, char *buf, int sz, IpAddress &from);
+static void htcpHandleNop(htcpDataHeader *, char *buf, int sz, Ip::Address &from);
 
-static void htcpHandleSet(htcpDataHeader *, char *buf, int sz, IpAddress &from);
+static void htcpHandleSet(htcpDataHeader *, char *buf, int sz, Ip::Address &from);
 
-static void htcpHandleTst(htcpDataHeader *, char *buf, int sz, IpAddress &from);
+static void htcpHandleTst(htcpDataHeader *, char *buf, int sz, Ip::Address &from);
 
 static void htcpRecv(int fd, void *data);
 
-static void htcpSend(const char *buf, int len, IpAddress &to);
+static void htcpSend(const char *buf, int len, Ip::Address &to);
 
-static void htcpTstReply(htcpDataHeader *, StoreEntry *, htcpSpecifier *, IpAddress &);
+static void htcpTstReply(htcpDataHeader *, StoreEntry *, htcpSpecifier *, Ip::Address &);
 
-static void htcpHandleTstRequest(htcpDataHeader *, char *buf, int sz, IpAddress &from);
+static void htcpHandleTstRequest(htcpDataHeader *, char *buf, int sz, Ip::Address &from);
 
-static void htcpHandleTstResponse(htcpDataHeader *, char *, int, IpAddress &);
+static void htcpHandleTstResponse(htcpDataHeader *, char *, int, Ip::Address &);
 
 static void
 htcpHexdump(const char *tag, const char *s, int sz)
@@ -306,19 +327,19 @@ htcpBuildAuth(char *buf, size_t buflen)
 {
     htcpAuthHeader auth;
     size_t copy_sz = 0;
-    assert(2 == sizeof(u_int16_t));
+    assert(2 == sizeof(uint16_t));
     auth.length = htons(2);
     copy_sz += 2;
     if (buflen < copy_sz)
         return -1;
-    xmemcpy(buf, &auth, copy_sz);
+    memcpy(buf, &auth, copy_sz);
     return copy_sz;
 }
 
 static ssize_t
 htcpBuildCountstr(char *buf, size_t buflen, const char *s)
 {
-    u_int16_t length;
+    uint16_t length;
     size_t len;
     int off = 0;
 
@@ -334,9 +355,9 @@ htcpBuildCountstr(char *buf, size_t buflen, const char *s)
 
     debugs(31, 3, "htcpBuildCountstr: TEXT = {" << (s ? s : "<NULL>") << "}");
 
-    length = htons((u_int16_t) len);
+    length = htons((uint16_t) len);
 
-    xmemcpy(buf + off, &length, 2);
+    memcpy(buf + off, &length, 2);
 
     off += 2;
 
@@ -344,7 +365,7 @@ htcpBuildCountstr(char *buf, size_t buflen, const char *s)
         return -1;
 
     if (len)
-        xmemcpy(buf + off, s, len);
+        memcpy(buf + off, s, len);
 
     off += len;
 
@@ -452,7 +473,7 @@ htcpBuildClrOpData(char *buf, size_t buflen, htcpStuff * stuff)
     case RR_REQUEST:
         debugs(31, 3, "htcpBuildClrOpData: RR_REQUEST");
         reason = htons((u_short)stuff->reason);
-        xmemcpy(buf, &reason, 2);
+        memcpy(buf, &reason, 2);
         return htcpBuildSpecifier(buf + 2, buflen - 2, stuff) + 2;
     case RR_RESPONSE:
         break;
@@ -509,7 +530,7 @@ htcpBuildData(char *buf, size_t buflen, htcpStuff * stuff)
 
     debugs(31, 3, "htcpBuildData: hdr.length = " << off);
 
-    hdr.length = (u_int16_t) off;
+    hdr.length = (uint16_t) off;
 
     hdr.opcode = stuff->op;
 
@@ -527,7 +548,7 @@ htcpBuildData(char *buf, size_t buflen, htcpStuff * stuff)
     hdr.msg_id = htonl(hdr.msg_id);
 
     if (!old_squid_format) {
-        xmemcpy(buf, &hdr, hdr_sz);
+        memcpy(buf, &hdr, hdr_sz);
     } else {
         htcpDataHeaderSquid hdrSquid;
         memset(&hdrSquid, 0, sizeof(hdrSquid));
@@ -536,7 +557,7 @@ htcpBuildData(char *buf, size_t buflen, htcpStuff * stuff)
         hdrSquid.response = hdr.response;
         hdrSquid.F1 = hdr.F1;
         hdrSquid.RR = hdr.RR;
-        xmemcpy(buf, &hdrSquid, hdr_sz);
+        memcpy(buf, &hdrSquid, hdr_sz);
     }
 
     debugs(31, 3, "htcpBuildData: size " << off);
@@ -576,7 +597,7 @@ htcpBuildPacket(char *buf, size_t buflen, htcpStuff * stuff)
     }
 
     off += s;
-    hdr.length = htons((u_int16_t) off);
+    hdr.length = htons((uint16_t) off);
     hdr.major = 0;
 
     if (old_squid_format)
@@ -584,7 +605,7 @@ htcpBuildPacket(char *buf, size_t buflen, htcpStuff * stuff)
     else
         hdr.minor = 1;
 
-    xmemcpy(buf, &hdr, hdr_sz);
+    memcpy(buf, &hdr, hdr_sz);
 
     debugs(31, 3, "htcpBuildPacket: size " << off);
 
@@ -593,7 +614,7 @@ htcpBuildPacket(char *buf, size_t buflen, htcpStuff * stuff)
 
 static void
 
-htcpSend(const char *buf, int len, IpAddress &to)
+htcpSend(const char *buf, int len, Ip::Address &to)
 {
     int x;
 
@@ -617,13 +638,13 @@ htcpSend(const char *buf, int len, IpAddress &to)
 
 void
 
-htcpSpecifier::setFrom (IpAddress &aSocket)
+htcpSpecifier::setFrom(Ip::Address &aSocket)
 {
     from = aSocket;
 }
 
 void
-htcpSpecifier::setDataHeader (htcpDataHeader *aDataHeader)
+htcpSpecifier::setDataHeader(htcpDataHeader *aDataHeader)
 {
     dhdr = aDataHeader;
 }
@@ -639,7 +660,7 @@ htcpFreeSpecifier(htcpSpecifier * s)
 static void
 htcpFreeDetail(htcpDetail * d)
 {
-    htcpDetailPool->free(d);
+    htcpDetailPool->freeOne(d);
 }
 
 /*
@@ -653,7 +674,7 @@ htcpUnpackSpecifier(char *buf, int sz)
     HttpRequestMethod method;
 
     /* Find length of METHOD */
-    u_int16_t l = ntohs(*(u_int16_t *) buf);
+    uint16_t l = ntohs(*(uint16_t *) buf);
     sz -= 2;
     buf += 2;
 
@@ -665,14 +686,12 @@ htcpUnpackSpecifier(char *buf, int sz)
 
     /* Set METHOD */
     s->method = buf;
-
     buf += l;
-
     sz -= l;
+    debugs(31, 6, "htcpUnpackSpecifier: METHOD (" << l << "/" << sz << ") '" << s->method << "'");
 
     /* Find length of URI */
-    l = ntohs(*(u_int16_t *) buf);
-
+    l = ntohs(*(uint16_t *) buf);
     sz -= 2;
 
     if (l > sz) {
@@ -683,19 +702,16 @@ htcpUnpackSpecifier(char *buf, int sz)
 
     /* Add terminating null to METHOD */
     *buf = '\0';
-
-    /* Set URI */
     buf += 2;
 
+    /* Set URI */
     s->uri = buf;
-
     buf += l;
-
     sz -= l;
+    debugs(31, 6, "htcpUnpackSpecifier: URI (" << l << "/" << sz << ") '" << s->uri << "'");
 
     /* Find length of VERSION */
-    l = ntohs(*(u_int16_t *) buf);
-
+    l = ntohs(*(uint16_t *) buf);
     sz -= 2;
 
     if (l > sz) {
@@ -706,19 +722,16 @@ htcpUnpackSpecifier(char *buf, int sz)
 
     /* Add terminating null to URI */
     *buf = '\0';
-
-    /* Set VERSION */
     buf += 2;
 
+    /* Set VERSION */
     s->version = buf;
-
     buf += l;
-
     sz -= l;
+    debugs(31, 6, "htcpUnpackSpecifier: VERSION (" << l << "/" << sz << ") '" << s->version << "'");
 
     /* Find length of REQ-HDRS */
-    l = ntohs(*(u_int16_t *) buf);
-
+    l = ntohs(*(uint16_t *) buf);
     sz -= 2;
 
     if (l > sz) {
@@ -729,15 +742,13 @@ htcpUnpackSpecifier(char *buf, int sz)
 
     /* Add terminating null to URI */
     *buf = '\0';
-
-    /* Set REQ-HDRS */
     buf += 2;
 
+    /* Set REQ-HDRS */
     s->req_hdrs = buf;
-
     buf += l;
-
     sz -= l;
+    debugs(31, 6, "htcpUnpackSpecifier: REQ-HDRS (" << l << "/" << sz << ") '" << s->req_hdrs << "'");
 
     debugs(31, 3, "htcpUnpackSpecifier: " << sz << " bytes left");
 
@@ -771,7 +782,7 @@ htcpUnpackDetail(char *buf, int sz)
     htcpDetail *d = static_cast<htcpDetail *>(htcpDetailPool->alloc());
 
     /* Find length of RESP-HDRS */
-    u_int16_t l = ntohs(*(u_int16_t *) buf);
+    uint16_t l = ntohs(*(uint16_t *) buf);
     sz -= 2;
     buf += 2;
 
@@ -789,7 +800,7 @@ htcpUnpackDetail(char *buf, int sz)
     sz -= l;
 
     /* Find length of ENTITY-HDRS */
-    l = ntohs(*(u_int16_t *) buf);
+    l = ntohs(*(uint16_t *) buf);
 
     sz -= 2;
 
@@ -812,7 +823,7 @@ htcpUnpackDetail(char *buf, int sz)
     sz -= l;
 
     /* Find length of CACHE-HDRS */
-    l = ntohs(*(u_int16_t *) buf);
+    l = ntohs(*(uint16_t *) buf);
 
     sz -= 2;
 
@@ -847,7 +858,7 @@ htcpUnpackDetail(char *buf, int sz)
 }
 
 static int
-htcpAccessCheck(acl_access * acl, htcpSpecifier * s, IpAddress &from)
+htcpAccessCheck(acl_access * acl, htcpSpecifier * s, Ip::Address &from)
 {
     /* default deny if no access list present */
     if (!acl)
@@ -861,7 +872,7 @@ htcpAccessCheck(acl_access * acl, htcpSpecifier * s, IpAddress &from)
 }
 
 static void
-htcpTstReply(htcpDataHeader * dhdr, StoreEntry * e, htcpSpecifier * spec, IpAddress &from)
+htcpTstReply(htcpDataHeader * dhdr, StoreEntry * e, htcpSpecifier * spec, Ip::Address &from)
 {
     htcpStuff stuff;
     static char pkt[8192];
@@ -950,7 +961,7 @@ htcpTstReply(htcpDataHeader * dhdr, StoreEntry * e, htcpSpecifier * spec, IpAddr
 
 static void
 
-htcpClrReply(htcpDataHeader * dhdr, int purgeSucceeded, IpAddress &from)
+htcpClrReply(htcpDataHeader * dhdr, int purgeSucceeded, Ip::Address &from)
 {
     htcpStuff stuff;
     static char pkt[8192];
@@ -987,7 +998,7 @@ htcpClrReply(htcpDataHeader * dhdr, int purgeSucceeded, IpAddress &from)
 
 static void
 
-htcpHandleNop(htcpDataHeader * hdr, char *buf, int sz, IpAddress &from)
+htcpHandleNop(htcpDataHeader * hdr, char *buf, int sz, Ip::Address &from)
 {
     debugs(31, 3, "htcpHandleNop: Unimplemented");
 }
@@ -1084,7 +1095,7 @@ htcpClrStore(const htcpSpecifier * s)
 
 static void
 
-htcpHandleTst(htcpDataHeader * hdr, char *buf, int sz, IpAddress &from)
+htcpHandleTst(htcpDataHeader * hdr, char *buf, int sz, Ip::Address &from)
 {
     debugs(31, 3, "htcpHandleTst: sz = " << sz);
 
@@ -1099,12 +1110,12 @@ HtcpReplyData::HtcpReplyData() : hdr(hoHtcpReply)
 
 static void
 
-htcpHandleTstResponse(htcpDataHeader * hdr, char *buf, int sz, IpAddress &from)
+htcpHandleTstResponse(htcpDataHeader * hdr, char *buf, int sz, Ip::Address &from)
 {
     htcpReplyData htcpReply;
     cache_key *key = NULL;
 
-    IpAddress *peer;
+    Ip::Address *peer;
     htcpDetail *d = NULL;
     char *t;
 
@@ -1171,7 +1182,7 @@ htcpHandleTstResponse(htcpDataHeader * hdr, char *buf, int sz, IpAddress &from)
 
 static void
 
-htcpHandleTstRequest(htcpDataHeader * dhdr, char *buf, int sz, IpAddress &from)
+htcpHandleTstRequest(htcpDataHeader * dhdr, char *buf, int sz, Ip::Address &from)
 {
     /* buf should be a SPECIFIER */
     htcpSpecifier *s;
@@ -1232,20 +1243,20 @@ htcpSpecifier::checkedHit(StoreEntry *e)
 
 static void
 
-htcpHandleMon(htcpDataHeader * hdr, char *buf, int sz, IpAddress &from)
+htcpHandleMon(htcpDataHeader * hdr, char *buf, int sz, Ip::Address &from)
 {
     debugs(31, 3, "htcpHandleMon: Unimplemented");
 }
 
 static void
 
-htcpHandleSet(htcpDataHeader * hdr, char *buf, int sz, IpAddress &from)
+htcpHandleSet(htcpDataHeader * hdr, char *buf, int sz, Ip::Address &from)
 {
     debugs(31, 3, "htcpHandleSet: Unimplemented");
 }
 
 static void
-htcpHandleClr(htcpDataHeader * hdr, char *buf, int sz, IpAddress &from)
+htcpHandleClr(htcpDataHeader * hdr, char *buf, int sz, Ip::Address &from)
 {
     htcpSpecifier *s;
     /* buf[0/1] is reserved and reason */
@@ -1340,7 +1351,7 @@ htcpForwardClr(char *buf, int sz)
  * hands it off to other functions to break apart message-specific data.
  */
 static void
-htcpHandleMsg(char *buf, int sz, IpAddress &from)
+htcpHandleMsg(char *buf, int sz, Ip::Address &from)
 {
     htcpHeader htcpHdr;
     htcpDataHeader hdr;
@@ -1354,7 +1365,7 @@ htcpHandleMsg(char *buf, int sz, IpAddress &from)
     }
 
     htcpHexdump("htcpHandle", buf, sz);
-    xmemcpy(&htcpHdr, buf, sizeof(htcpHeader));
+    memcpy(&htcpHdr, buf, sizeof(htcpHeader));
     htcpHdr.length = ntohs(htcpHdr.length);
 
     if (htcpHdr.minor == 0)
@@ -1388,10 +1399,10 @@ htcpHandleMsg(char *buf, int sz, IpAddress &from)
     }
 
     if (!old_squid_format) {
-        xmemcpy(&hdr, hbuf, sizeof(hdr));
+        memcpy(&hdr, hbuf, sizeof(hdr));
     } else {
         htcpDataHeaderSquid hdrSquid;
-        xmemcpy(&hdrSquid, hbuf, sizeof(hdrSquid));
+        memcpy(&hdrSquid, hbuf, sizeof(hdrSquid));
         hdr.length = hdrSquid.length;
         hdr.opcode = hdrSquid.opcode;
         hdr.response = hdrSquid.response;
@@ -1460,7 +1471,7 @@ htcpRecv(int fd, void *data)
 {
     static char buf[8192];
     int len;
-    static IpAddress from;
+    static Ip::Address from;
 
     /* Receive up to 8191 bytes, leaving room for a null */
 
@@ -1490,28 +1501,41 @@ htcpInit(void)
         return;
     }
 
-    IpAddress incomingAddr = Config.Addrs.udp_incoming;
+    Ip::Address incomingAddr = Config.Addrs.udp_incoming;
     incomingAddr.SetPort(Config.Port.htcp);
 
-    enter_suid();
-    htcpInSocket = comm_open_listener(SOCK_DGRAM,
-                                      IPPROTO_UDP,
-                                      incomingAddr,
-                                      COMM_NONBLOCKING,
-                                      "HTCP Socket");
-    leave_suid();
+    if (!Ip::EnableIpv6 && !incomingAddr.SetIPv4()) {
+        debugs(31, DBG_CRITICAL, "ERROR: IPv6 is disabled. " << incomingAddr << " is not an IPv4 address.");
+        fatal("HTCP port cannot be opened.");
+    }
+    /* split-stack for now requires default IPv4-only HTCP */
+    if (Ip::EnableIpv6&IPV6_SPECIAL_SPLITSTACK && incomingAddr.IsAnyAddr()) {
+        incomingAddr.SetIPv4();
+    }
 
-    if (htcpInSocket < 0)
-        fatal("Cannot open HTCP Socket");
+    AsyncCall::Pointer call = asyncCall(31, 2,
+                                        "htcpIncomingConnectionOpened",
+                                        HtcpListeningStartedDialer(&htcpIncomingConnectionOpened));
 
-    commSetSelect(htcpInSocket, COMM_SELECT_READ, htcpRecv, NULL, 0);
-
-    debugs(31, 1, "Accepting HTCP messages on port " << Config.Port.htcp << ", FD " << htcpInSocket << ".");
+    Ipc::StartListening(SOCK_DGRAM,
+                        IPPROTO_UDP,
+                        incomingAddr,
+                        COMM_NONBLOCKING,
+                        Ipc::fdnInHtcpSocket, call);
 
     if (!Config.Addrs.udp_outgoing.IsNoAddr()) {
-        IpAddress outgoingAddr = Config.Addrs.udp_outgoing;
+        Ip::Address outgoingAddr = Config.Addrs.udp_outgoing;
         outgoingAddr.SetPort(Config.Port.htcp);
 
+        if (!Ip::EnableIpv6 && !outgoingAddr.SetIPv4()) {
+            debugs(31, DBG_CRITICAL, "ERROR: IPv6 is disabled. " << outgoingAddr << " is not an IPv4 address.");
+            fatal("HTCP port cannot be opened.");
+        }
+        /* split-stack for now requires default IPv4-only HTCP */
+        if (Ip::EnableIpv6&IPV6_SPECIAL_SPLITSTACK && outgoingAddr.IsAnyAddr()) {
+            outgoingAddr.SetIPv4();
+        }
+
         enter_suid();
         htcpOutSocket = comm_open_listener(SOCK_DGRAM,
                                            IPPROTO_UDP,
@@ -1528,8 +1552,6 @@ htcpInit(void)
         debugs(31, 1, "Outgoing HTCP messages on port " << Config.Port.htcp << ", FD " << htcpOutSocket << ".");
 
         fd_note(htcpInSocket, "Incoming HTCP socket");
-    } else {
-        htcpOutSocket = htcpInSocket;
     }
 
     if (!htcpDetailPool) {
@@ -1537,6 +1559,22 @@ htcpInit(void)
     }
 }
 
+static void
+htcpIncomingConnectionOpened(int fd, int errNo)
+{
+    htcpInSocket = fd;
+
+    if (htcpInSocket < 0)
+        fatal("Cannot open HTCP Socket");
+
+    commSetSelect(htcpInSocket, COMM_SELECT_READ, htcpRecv, NULL, 0);
+
+    debugs(31, 1, "Accepting HTCP messages on port " << Config.Port.htcp << ", FD " << htcpInSocket << ".");
+
+    if (Config.Addrs.udp_outgoing.IsNoAddr())
+        htcpOutSocket = htcpInSocket;
+}
+
 int
 htcpQuery(StoreEntry * e, HttpRequest * req, peer * p)
 {
@@ -1711,7 +1749,7 @@ htcpSocketClose(void)
 }
 
 static void
-htcpLogHtcp(IpAddress &caddr, int opcode, log_type logcode, const char *url)
+htcpLogHtcp(Ip::Address &caddr, int opcode, log_type logcode, const char *url)
 {
     AccessLogEntry al;
     if (LOG_TAG_NONE == logcode)
