@@ -34,10 +34,10 @@
 #include "ProtoPort.h"
 #include "acl/FilledChecklist.h"
 #include "event.h"
-#include "CacheManager.h"
 #include "htcp.h"
 #include "HttpRequest.h"
 #include "ICP.h"
+#include "ip/tools.h"
 #include "MemObject.h"
 #include "PeerDigest.h"
 #include "PeerSelectState.h"
@@ -47,6 +47,7 @@
 #include "icmp/net_db.h"
 #include "ip/Address.h"
 #include "ip/tools.h"
+#include "mgr/Registration.h"
 
 /* count mcast group peers every 15 minutes */
 #define MCAST_COUNT_RATE 900
@@ -320,7 +321,7 @@ getRoundRobinParent(HttpRequest * request)
             if (p->weight == q->weight) {
                 if (q->rr_count < p->rr_count)
                     continue;
-            } else if ( (double) q->rr_count / q->weight < (double) p->rr_count / p->weight) {
+            } else if ( ((double) q->rr_count / q->weight) < ((double) p->rr_count / p->weight)) {
                 continue;
             }
         }
@@ -540,15 +541,14 @@ neighborRemove(peer * target)
 static void
 neighborsRegisterWithCacheManager()
 {
-    CacheManager *manager = CacheManager::GetInstance();
-    manager->registerAction("server_list",
-                            "Peer Cache Statistics",
-                            neighborDumpPeers, 0, 1);
+    Mgr::RegisterAction("server_list",
+                        "Peer Cache Statistics",
+                        neighborDumpPeers, 0, 1);
 
     if (theInIcpConnection >= 0) {
-        manager->registerAction("non_peers",
-                                "List of Unknown sites sending ICP messages",
-                                neighborDumpNonPeers, 0, 1);
+        Mgr::RegisterAction("non_peers",
+                            "List of Unknown sites sending ICP messages",
+                            neighborDumpNonPeers, 0, 1);
     }
 }
 
@@ -1204,17 +1204,15 @@ peerDestroy(void *data)
 {
     peer *p = (peer *)data;
 
-    struct _domain_ping *l = NULL;
-
-    struct _domain_ping *nl = NULL;
-
     if (p == NULL)
         return;
 
-    for (l = p->peer_domain; l; l = nl) {
+    struct _domain_ping *nl = NULL;
+
+    for (struct _domain_ping *l = p->peer_domain; l; l = nl) {
         nl = l->next;
         safe_free(l->domain);
-        safe_free(l);
+        xfree(l);
     }
 
     safe_free(p->host);
@@ -1620,11 +1618,9 @@ dump_peer_options(StoreEntry * sentry, peer * p)
     if (p->options.no_netdb_exchange)
         storeAppendPrintf(sentry, " no-netdb-exchange");
 
-#if DELAY_POOLS
-
+#if USE_DELAY_POOLS
     if (p->options.no_delay)
         storeAppendPrintf(sentry, " no-delay");
-
 #endif
 
     if (p->login)
@@ -1759,7 +1755,7 @@ dump_peers(StoreEntry * sentry, peer * peers)
 
         if (e->stats.last_connect_failure) {
             storeAppendPrintf(sentry, "Last failed connect() at: %s\n",
-                              mkhttpdlogtime(&(e->stats.last_connect_failure)));
+                              Time::FormatHttpd(e->stats.last_connect_failure));
         }
 
         if (e->peer_domain != NULL) {
