@@ -35,25 +35,24 @@
 
 #include "squid.h"
 #include "auth/UserRequest.h"
-#include "CacheManager.h"
+#include "mgr/Registration.h"
 #include "Store.h"
 #include "fde.h"
 #include "client_side_request.h"
 #include "acl/Checklist.h"
 #include "HttpRequest.h"
 #include "client_side.h"
-#include "client_side_reply.h"
 #include "helper.h"
 #include "rfc1738.h"
-
-/// url maximum lengh + extra informations passed to redirector
-#define MAX_REDIRECTOR_REQUEST_STRLEN (MAX_URL + 1024)
+#if USE_SSL
+#include "ssl/support.h"
+#endif
 
 typedef struct {
     void *data;
     char *orig_url;
 
-    IpAddress client_addr;
+    Ip::Address client_addr;
     const char *client_ident;
     const char *method_s;
     RH *handler;
@@ -118,9 +117,7 @@ redirectStart(ClientHttpRequest * http, RH * handler, void *data)
     ConnStateData * conn = http->getConn();
     redirectStateData *r = NULL;
     const char *fqdn;
-    char buf[MAX_REDIRECTOR_REQUEST_STRLEN];
-    int sz;
-    http_status status;
+    char buf[8192];
     char claddr[MAX_IPSTRLEN];
     char myaddr[MAX_IPSTRLEN];
     assert(http);
@@ -141,28 +138,21 @@ redirectStart(ClientHttpRequest * http, RH * handler, void *data)
     else
         r->client_addr.SetNoAddr();
     r->client_ident = NULL;
-    if (http->request->auth_user_request != NULL) {
+
+    if (http->request->auth_user_request != NULL)
         r->client_ident = http->request->auth_user_request->username();
-        debugs(61, 5, HERE << "auth-user=" << (r->client_ident?r->client_ident:"NULL"));
-    }
-
-    // HttpRequest initializes with null_string. So we must check both defined() and size()
-    if (!r->client_ident && http->request->extacl_user.defined() && http->request->extacl_user.size()) {
+    else if (http->request->extacl_user.defined()) {
         r->client_ident = http->request->extacl_user.termedBuf();
-        debugs(61, 5, HERE << "acl-user=" << (r->client_ident?r->client_ident:"NULL"));
     }
 
-    if (!r->client_ident && (conn != NULL && conn->rfc931[0])) {
+    if (!r->client_ident && (conn != NULL && conn->rfc931[0]))
         r->client_ident = conn->rfc931;
-        debugs(61, 5, HERE << "ident-user=" << (r->client_ident?r->client_ident:"NULL"));
-    }
 
 #if USE_SSL
 
-    if (!r->client_ident && conn != NULL) {
+    if (!r->client_ident && conn != NULL)
         r->client_ident = sslGetUserEmail(fd_table[conn->fd].ssl);
-        debugs(61, 5, HERE << "ssl-user=" << (r->client_ident?r->client_ident:"NULL"));
-    }
+
 #endif
 
     if (!r->client_ident)
@@ -177,51 +167,22 @@ redirectStart(ClientHttpRequest * http, RH * handler, void *data)
     if ((fqdn = fqdncache_gethostbyaddr(r->client_addr, 0)) == NULL)
         fqdn = dash_str;
 
-    sz = snprintf(buf, MAX_REDIRECTOR_REQUEST_STRLEN, "%s %s/%s %s %s myip=%s myport=%d\n",
-                  r->orig_url,
-                  r->client_addr.NtoA(claddr,MAX_IPSTRLEN),
-                  fqdn,
-                  r->client_ident[0] ? rfc1738_escape(r->client_ident) : dash_str,
-                  r->method_s,
-                  http->request->my_addr.NtoA(myaddr,MAX_IPSTRLEN),
-                  http->request->my_addr.GetPort());
+    snprintf(buf, 8192, "%s %s/%s %s %s myip=%s myport=%d\n",
+             r->orig_url,
+             r->client_addr.NtoA(claddr,MAX_IPSTRLEN),
+             fqdn,
+             r->client_ident[0] ? rfc1738_escape(r->client_ident) : dash_str,
+             r->method_s,
+             http->request->my_addr.NtoA(myaddr,MAX_IPSTRLEN),
+             http->request->my_addr.GetPort());
 
-    if ((sz<=0) || (sz>=MAX_REDIRECTOR_REQUEST_STRLEN)) {
-        if (sz<=0) {
-            status = HTTP_INTERNAL_SERVER_ERROR;
-            debugs(61, DBG_CRITICAL, "ERROR: Gateway Failure. Can not build request to be passed to redirector. Request ABORTED.");
-        } else {
-            status = HTTP_REQUEST_URI_TOO_LARGE;
-            debugs(61, DBG_CRITICAL, "ERROR: Gateway Failure. Request passed to redirector exceeds MAX_REDIRECTOR_REQUEST_STRLEN (" << MAX_REDIRECTOR_REQUEST_STRLEN << "). Request ABORTED.");
-        }
-
-        clientStreamNode *node = (clientStreamNode *)http->client_stream.tail->prev->data;
-        clientReplyContext *repContext = dynamic_cast<clientReplyContext *>(node->data.getRaw());
-        assert (repContext);
-        IpAddress tmpnoaddr;
-        tmpnoaddr.SetNoAddr();
-        repContext->setReplyToError(ERR_GATEWAY_FAILURE, status,
-                                    http->request->method, NULL,
-                                    http->getConn() != NULL ? http->getConn()->peer : tmpnoaddr,
-                                    http->request,
-                                    NULL,
-                                    http->getConn() != NULL && http->getConn()->auth_user_request != NULL ?
-                                    http->getConn()->auth_user_request : http->request->auth_user_request);
-
-        node = (clientStreamNode *)http->client_stream.tail->data;
-        clientStreamRead(node, http, node->readBuffer);
-        return;
-    }
-
-    debugs(61,6, HERE << "sending '" << buf << "' to the helper");
     helperSubmit(redirectors, buf, redirectHandleReply, r);
 }
 
 static void
 redirectRegisterWithCacheManager(void)
 {
-    CacheManager::GetInstance()->
-    registerAction("redirector", "URL Redirector Stats", redirectStats, 0, 1);
+    Mgr::RegisterAction("redirector", "URL Redirector Stats", redirectStats, 0, 1);
 }
 
 void
@@ -235,13 +196,11 @@ redirectInit(void)
         return;
 
     if (redirectors == NULL)
-        redirectors = helperCreate("redirector");
+        redirectors = new helper("redirector");
 
     redirectors->cmdline = Config.Program.redirect;
 
-    redirectors->n_to_start = Config.redirectChildren;
-
-    redirectors->concurrency = Config.redirectConcurrency;
+    redirectors->childs = Config.redirectChildren;
 
     redirectors->ipc_type = IPC_STREAM;
 
@@ -264,7 +223,6 @@ redirectShutdown(void)
     if (!shutting_down)
         return;
 
-    helperFree(redirectors);
-
+    delete redirectors;
     redirectors = NULL;
 }
