@@ -154,9 +154,6 @@ using namespace Squid;
 #endif
 
 #include "md5.h"
-#if USE_SSL
-#include "ssl_support.h"
-#endif
 #if SQUID_SNMP
 #include "cache_snmp.h"
 #endif
@@ -166,7 +163,7 @@ using namespace Squid;
 #include "enums.h"
 #include "typedefs.h"
 #include "util.h"
-#include "profiling.h"
+#include "profiler/Profiler.h"
 #include "MemPool.h"
 #include "ip/Address.h"
 #include "structs.h"
