@@ -7,7 +7,6 @@ struct SslErrorDetailEntry {
     const char *detail;
 };
 
-static const char *SslErrorDetailDefaultStr = "SSL certificate validation error (%err_name): %ssl_subject";
 // TODO: optimize by replacing with std::map or similar
 static SslErrorDetailEntry TheSslDetailMap[] = {
     {  SQUID_X509_V_ERR_DOMAIN_MISMATCH,
@@ -21,7 +20,7 @@ static SslErrorDetailEntry TheSslDetailMap[] = {
       "%err_name: SSL Certficate is not valid before: %ssl_notbefore" },
     { X509_V_ERR_ERROR_IN_CERT_NOT_BEFORE_FIELD,
       "X509_V_ERR_ERROR_IN_CERT_NOT_BEFORE_FIELD",
-      "%err_name: SSL Certificate has invalid start date (the 'not before' field): %ssl_subject" },
+      "%err_name: SSL Certificate has invalid start date (the 'not before' field): %subject" },
     { X509_V_ERR_CERT_HAS_EXPIRED,
       "X509_V_ERR_CERT_HAS_EXPIRED",
       "%err_name: SSL Certificate expired on %ssl_notafter" },
@@ -78,9 +77,7 @@ static const char *getErrorDetail(Ssl::ssl_error_t value)
             return TheSslDetailMap[i].detail;
     }
 
-    // we must always return something because ErrorDetail::buildDetail
-    // will hit an assertion
-    return SslErrorDetailDefaultStr;
+    return NULL;
 }
 
 Ssl::ErrorDetail::err_frm_code Ssl::ErrorDetail::ErrorFormatingCodes[] = {
@@ -179,12 +176,9 @@ const char *Ssl::ErrorDetail::notafter() const
  */
 const char *Ssl::ErrorDetail::err_code() const
 {
-    static char tmpBuffer[64];
     const char *err = getErrorName(error_no);
-    if (!err) {
-        snprintf(tmpBuffer, 64, "%d", (int)error_no);
-        err = tmpBuffer;
-    }
+    if (!err)
+        return "[Not available]";
     return err;
 }
 
@@ -226,7 +220,9 @@ void Ssl::ErrorDetail::buildDetail() const
     char const *t;
     int code_len = 0;
 
-    assert(s);
+    if (!s)  //May be add a default detail string?
+        return;
+
     while ((p = strchr(s, '%'))) {
         errDetailStr.append(s, p - s);
         code_len = convert(++p, &t);
