@@ -29,7 +29,6 @@
 #ifndef SQUID_STRUCTS_H
 #define SQUID_STRUCTS_H
 
-#include "config.h"
 #include "RefCount.h"
 #include "cbdata.h"
 #include "dlink.h"
@@ -41,6 +40,10 @@
 /* for ICP_END */
 #include "icp_opcode.h"
 
+#if USE_SSL
+#include <openssl/ssl.h>
+#endif
+
 #define PEER_MULTICAST_SIBLINGS 1
 
 struct acl_name_list {
@@ -124,7 +127,7 @@ struct relist {
     relist *next;
 };
 
-#if DELAY_POOLS
+#if USE_DELAY_POOLS
 #include "DelayConfig.h"
 #include "ClientDelayConfig.h"
 #endif
@@ -196,6 +199,7 @@ struct SquidConfig {
     size_t maxRequestHeaderSize;
     int64_t maxRequestBodySize;
     int64_t maxChunkedRequestBodySize;
+    size_t maxRequestBufferSize;
     size_t maxReplyHeaderSize;
     acl_size_t *ReplyBodySize;
 
@@ -257,27 +261,10 @@ struct SquidConfig {
     struct {
         char *store;
         char *swap;
-#if USE_USERAGENT_LOG
-
-        char *useragent;
-#endif
-#if USE_REFERER_LOG
-
-        char *referer;
-#endif
-#if WIP_FWD_LOG
-
-        char *forward;
-#endif
-
-        logformat *logformats;
-
         customlog *accesslogs;
-
 #if ICAP_CLIENT
         customlog *icaplogs;
 #endif
-
         int rotateNumber;
     } Log;
     char *adminEmail;
@@ -533,7 +520,7 @@ struct SquidConfig {
     struct {
         int64_t limit;
     } MemPools;
-#if DELAY_POOLS
+#if USE_DELAY_POOLS
 
     DelayConfig Delay;
     ClientDelayConfig ClientDelay;
@@ -886,7 +873,7 @@ struct peer {
         unsigned int htcp_forward_clr:1;
 #endif
         unsigned int no_netdb_exchange:1;
-#if DELAY_POOLS
+#if USE_DELAY_POOLS
         unsigned int no_delay:1;
 #endif
         unsigned int allow_miss:1;
@@ -1311,23 +1298,17 @@ struct _store_rebuild_data {
     int zero_object_sz;
 };
 
-class logformat_token;
-
-struct _logformat {
-    char *name;
-    logformat_token *format;
-    logformat *next;
-};
-
 class Logfile;
+class logformat;
 
+#include "log/Formats.h"
 struct _customlog {
     char *filename;
     ACLList *aclList;
     logformat *logFormat;
     Logfile *logfile;
     customlog *next;
-    customlog_type type;
+    Log::Format::log_type type;
 };
 
 #endif /* SQUID_STRUCTS_H */
